@@ -13,7 +13,11 @@
  *	the callback RPC interface.
  */
 
-#include "../afs/param.h"       /*Should be always first*/
+#include <afsconfig.h>
+#include "../afs/param.h"
+
+RCSID("$Header: /cvs/openafs/src/afs/afs_callback.c,v 1.6.2.5 2002/04/21 20:23:56 zacheiss Exp $");
+
 #include "../afs/sysincludes.h" /*Standard vendor system headers*/
 #include "../afs/afsincludes.h" /*AFS-based standard headers*/
 #include "../afs/afs_stats.h"	/*Cache Manager stats*/
@@ -153,7 +157,7 @@ int SRXAFSCB_GetCE(a_call, a_index, a_result)
     a_result->DataVersion = hgetlo(tvc->m.DataVersion);
     a_result->callback = afs_data_pointer_to_int32(tvc->callback);		/* XXXX Now a pointer; change it XXXX */
     a_result->cbExpires = tvc->cbExpires;
-    a_result->refCount = tvc->vrefCount;
+    a_result->refCount = VREFCOUNT(tvc);
     a_result->opens = tvc->opens;
     a_result->writers = tvc->execsOrWriters;
     a_result->mvstat = tvc->mvstat;
@@ -235,9 +239,15 @@ int SRXAFSCB_GetLock (a_call, a_index, a_result)
 	a_result->lock.exclLocked = ((struct afs_lock *)(tl->addr))->excl_locked;
 	a_result->lock.readersReading = ((struct afs_lock *)(tl->addr))->readers_reading;
 	a_result->lock.numWaiting = ((struct afs_lock *)(tl->addr))->num_waiting;
+#ifdef INSTRUMENT_LOCKS
 	a_result->lock.pid_last_reader = ((struct afs_lock *)(tl->addr))->pid_last_reader;
 	a_result->lock.pid_writer = ((struct afs_lock *)(tl->addr))->pid_writer;
 	a_result->lock.src_indicator = ((struct afs_lock *)(tl->addr))->src_indicator;
+#else
+	a_result->lock.pid_last_reader = 0;
+	a_result->lock.pid_writer = 0;
+	a_result->lock.src_indicator = 0;
+#endif
 	code = 0;
     }
 
@@ -715,7 +725,7 @@ int SRXAFSCB_GetXStats(a_call, a_clientVersionNum, a_collectionNumber, a_srvVers
 	 */
 	dataBytes = sizeof(struct afs_CMStats);
 	dataBuffP = (afs_int32 *)afs_osi_Alloc(dataBytes);
-	bcopy((char *)&afs_cmstats, (char *)dataBuffP, dataBytes);
+	memcpy((char *)dataBuffP, (char *)&afs_cmstats, dataBytes);
 	a_dataP->AFSCB_CollData_len = dataBytes>>2;
 	a_dataP->AFSCB_CollData_val = dataBuffP;
 	break;
@@ -735,7 +745,7 @@ int SRXAFSCB_GetXStats(a_call, a_clientVersionNum, a_collectionNumber, a_srvVers
 	afs_CountServers();
 	dataBytes = sizeof(afs_stats_cmperf);
 	dataBuffP = (afs_int32 *)afs_osi_Alloc(dataBytes);
-	bcopy((char *)&afs_stats_cmperf, (char *)dataBuffP, dataBytes);
+	memcpy((char *)dataBuffP, (char *)&afs_stats_cmperf, dataBytes);
 	a_dataP->AFSCB_CollData_len = dataBytes>>2;
 	a_dataP->AFSCB_CollData_val = dataBuffP;
 	break;
@@ -753,14 +763,12 @@ int SRXAFSCB_GetXStats(a_call, a_clientVersionNum, a_collectionNumber, a_srvVers
 	 */
 	afs_stats_cmperf.numPerfCalls++;
 	afs_CountServers();
-	bcopy((char *)(&afs_stats_cmperf),
-	      (char *)(&(afs_stats_cmfullperf.perf)),
-	      sizeof(struct afs_stats_CMPerf));
+	memcpy((char *)(&(afs_stats_cmfullperf.perf)), (char *)(&afs_stats_cmperf), sizeof(struct afs_stats_CMPerf));
 	afs_stats_cmfullperf.numFullPerfCalls++;
 
 	dataBytes = sizeof(afs_stats_cmfullperf);
 	dataBuffP = (afs_int32 *)afs_osi_Alloc(dataBytes);
-	bcopy((char *)(&afs_stats_cmfullperf), (char *)dataBuffP, dataBytes);
+	memcpy((char *)dataBuffP, (char *)(&afs_stats_cmfullperf), dataBytes);
 	a_dataP->AFSCB_CollData_len = dataBytes>>2;
 	a_dataP->AFSCB_CollData_val = dataBuffP;
 	break;
@@ -1122,36 +1130,44 @@ int SRXAFSCB_GetCellServDB(
     afs_int32 i, j;
     struct cell *tcell;
     struct afs_q *cq, *tq;
-    char *t_name;
+    char *t_name, *p_name = NULL;
 
 #ifdef RX_ENABLE_LOCKS
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
     AFS_STATCNT(SRXAFSCB_GetCellServDB);
 
-    t_name = (char *)rxi_Alloc(AFSNAMEMAX);
+    memset(a_hosts, 0, AFSMAXCELLHOSTS * sizeof(afs_int32));
+
+    /* search the list for the cell with this index */
+    ObtainReadLock(&afs_xcell);
+
+    tcell = afs_GetCellByIndex(a_index, READ_LOCK, 0);
+
+    if (!tcell) {
+      i = 0;
+    } else {
+      p_name = tcell->cellName;
+      for (j = 0 ; j < AFSMAXCELLHOSTS && tcell->cellHosts[j] ; j++) {
+	a_hosts[j] = ntohl(tcell->cellHosts[j]->addr->sa_ip);
+      }
+      i = strlen(p_name);
+      afs_PutCell(tcell, READ_LOCK);
+    }
+
+    t_name = (char *)rxi_Alloc(i+1);
     if (t_name == NULL) {
+        ReleaseReadLock(&afs_xcell);
 #ifdef RX_ENABLE_LOCKS
 	AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
 	return ENOMEM;
     }
 
-    t_name[0] = '\0';
-    bzero(a_hosts, AFSMAXCELLHOSTS * sizeof(afs_int32));
+    t_name[i] = '\0';
+    if (p_name)
+	memcpy(t_name, p_name, i);
 
-    /* search the list for the cell with this index */
-    ObtainReadLock(&afs_xcell);
-    for (i=0, cq = CellLRU.next; cq != &CellLRU && i<= a_index; cq = tq, i++) {
-        tq = QNext(cq);
-	if (i == a_index) {
-	    tcell = QTOC(cq);
-	    strcpy(t_name, tcell->cellName);
-	    for (j = 0 ; j < AFSMAXCELLHOSTS && tcell->cellHosts[j] ; j++) {
-		a_hosts[j] = ntohl(tcell->cellHosts[j]->addr->sa_ip);
-	    }
-	}
-    }
     ReleaseReadLock(&afs_xcell);
 
 #ifdef RX_ENABLE_LOCKS
@@ -1187,40 +1203,50 @@ int SRXAFSCB_GetLocalCell(
     struct rx_call *a_call,
     char **a_name)
 {
+    int plen;
     struct cell *tcell;
     struct afs_q *cq, *tq;
-    char *t_name;
+    char *t_name, *p_name = NULL;
 
 #ifdef RX_ENABLE_LOCKS
     AFS_GLOCK();
 #endif /* RX_ENABLE_LOCKS */
     AFS_STATCNT(SRXAFSCB_GetLocalCell);
 
-    t_name = (char *)rxi_Alloc(AFSNAMEMAX);
+    /* Search the list for the primary cell. Cell number 1 is only
+     * the primary cell is when no other cell is explicitly marked as
+     * the primary cell.  */
+    ObtainReadLock(&afs_xcell);
+
+    for (cq = CellLRU.next; cq != &CellLRU; cq = tq) {
+        tq = QNext(cq);
+	tcell = QTOC(cq);
+	if (tcell->states & CPrimary) {
+	    p_name = tcell->cellName;
+	    break;
+	}
+        if (tcell->cell == 1) {
+	    p_name = tcell->cellName;
+	}
+    }
+
+    if (p_name)
+	plen = strlen(p_name);
+    else
+	plen = 0;
+    t_name = (char *)rxi_Alloc(plen+1);
     if (t_name == NULL) {
+        ReleaseReadLock(&afs_xcell);
 #ifdef RX_ENABLE_LOCKS
 	AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
 	return ENOMEM;
     }
 
-    t_name[0] = '\0';
+    t_name[plen] = '\0';
+    if (p_name)
+	memcpy(t_name, p_name, plen);
 
-    /* Search the list for the primary cell. Cell number 1 is only
-     * the primary cell is when no other cell is explicitly marked as
-     * the primary cell.  */
-    ObtainReadLock(&afs_xcell);
-    for (cq = CellLRU.next; cq != &CellLRU; cq = tq) {
-        tq = QNext(cq);
-	tcell = QTOC(cq);
-	if (tcell->states & CPrimary) {
-	    strcpy(t_name, tcell->cellName);
-	    break;
-	}
-        if (tcell->cell == 1) {
-	    strcpy(t_name, tcell->cellName);
-	}
-    }
     ReleaseReadLock(&afs_xcell);
 
 #ifdef RX_ENABLE_LOCKS
